@@ -3,8 +3,8 @@ $\ = "\n\n";
 $_ = 'value';
 
 my $map = new Unicode::Map();
-die <DATA> unless $_ eq 'value';
-print "The Module 'Unicode::Map' OK!";
+unless($_ eq 'value'){ print <DATA> }
+else  { print "The Module 'Unicode::Map' OK!" }
 
 
 
@@ -12,7 +12,6 @@ use Unicode::Lite;
 $Unicode::Lite::TEST = 1;
 
 
-
 $_ = "drben, Strae";
 
 print convert( 'latin1', 'utf8' );
